
# fonts - a Sketch-specific version of the Impression module's font handling
#         module
#
# Copyright (c) 2002-2003, David Boddie
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

"""
fonts.py

Font support for outline fonts.
"""

import types, sys

from Sketch import GetFont

#import fontsubs
import imp, os

def include(name, as_module, path = None):

    module_path, file = os.path.split(globals()["__file__"])
    
    if path is not None:
    
        module_path = os.path.join(module_path, path)
    
    new_path = os.path.join(module_path, name + os.extsep + "py")
    
    code_string = open(new_path, "r").read()
    
    code = compile(code_string, new_path, "exec")
    
    if as_module == 0:
    
        exec code in globals()
    
    else:
    
        module = imp.new_module(name)
        exec code in module.__dict__
        return module

fontsubs = include("fontsubs", 1, path = "../Common")

# An exception class for font related errors
class FontError(Exception):

    pass



# Find the font substitution definitions.
res = fontsubs.locate_resources()

# Create a substitution object.
subs = fontsubs.Substitutions(res)


class Font:
    """instance = Font(name, x_size, y_size)
    Create an instance of a font having the specified name and size, measured
    in points.
    """

    def __init__(self, name, x_size, y_size):

        # Find the relevant font and obtain a font handle
        # Sizes are in millipoints
        self.x_size = x_size
        self.y_size = y_size

        # The font is found via the substitution mechanism.
        self.actual_font = GetFont(subs[name])
    
    def string_width(self, text):
        """width = string_width(string)
        Return the width in millipoints this string would have if it
        were displayed using this font object.
        """
        # Measure the width of a string in millipoints.
        return self.actual_font.TextCoordBox(text, self.x_size)[2]
